/*
Copyright 2004-2005 University Corporation for Advanced Internet Development, Inc.
Copyright 2004-2005 The University Of Bristol

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package edu.internet2.middleware.grouper.ui.actions;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.sun.org.apache.xalan.internal.xsltc.runtime.Hashtable;

import edu.internet2.middleware.grouper.*;
import edu.internet2.middleware.grouper.ui.GroupOrStem;
import edu.internet2.middleware.grouper.ui.Message;

/**
 

 * @author Gary Brown.
 * @version $Id: JoinGroupAction.java,v 1.3 2006/02/24 13:40:58 isgwb Exp $
 */
public class ExportXmlAction extends GrouperCapableAction {

	

	public ActionForward grouperExecute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response,
			HttpSession session, GrouperSession grouperSession)
			throws Exception {
		Properties defaultProps = new Properties();
		defaultProps.load(Group.class.getResourceAsStream("export.properties"));
		Properties props  = new Properties(defaultProps);
		Enumeration en = request.getParameterNames();
		Map params = request.getParameterMap();
		if(params.size()<2) {
			String[] paramList = new String[] {"id","name","exclusive","children","relative","includeParent"};
			request.setAttribute("exportParamList",paramList);
			if(session.getAttribute("lastParams")==null) {
				session.setAttribute("lastParams",new HashMap());
			}
			if(session.getAttribute("exportProps")==null) {
				session.setAttribute("exportProps",properties2map(defaultProps));
			}
			return mapping.findForward("ExportXmlForm");
		}
		
		
		
		
		String key;
		while(en.hasMoreElements()) {
			key = en.nextElement().toString();
			if(key.startsWith("export.")) {
				props.setProperty(key, request.getParameter(key));
			}
		}
		
		XmlExporter xe = new XmlExporter(grouperSession,props);
		response.setContentType("text/xml");
		PrintWriter pw = response.getWriter();
		session.setAttribute("exportProps",properties2map(props));
		
		session.setAttribute("lastParams",params);
		
		String doSearch = request.getParameter("submit.search");
		if(!isEmpty(doSearch)) {
			return search(request,response,session,grouperSession,xe,pw);
		}
		
		
		String id=request.getParameter("id");
		String name=request.getParameter("name");
		String exclusive=request.getParameter("exclusive");
		String children=request.getParameter("children");
		String relative=request.getParameter("relative");
		String includeParent=request.getParameter("includeParent");
		GroupOrStem gos=null;
		if(!isEmpty(id)) {
			gos=GroupOrStem.findByID(grouperSession, id);
		}else if(!isEmpty(name)) {
			gos=GroupOrStem.findByName(grouperSession, name);
		}

		
		
		if(gos.isGroup()) {
			xe.export(pw, gos.getGroup(), !isEmpty(relative), !isEmpty(includeParent));
		}else{
			if(isEmpty(children)) {
				if(isEmpty(exclusive)) {
					xe.export(pw, gos.getStem(), !isEmpty(relative));
				}else{
					List stem = new ArrayList();
					stem.add(gos.getStem());
					xe.export(pw, stem, "Export single stem");
					
					
				}
			}else{
				List childs = new ArrayList();
				Stem stem = gos.getStem();
				childs.addAll(stem.getChildStems());
				childs.addAll(stem.getChildGroups());
				xe.export(pw, childs, "Child stems and groups for " + stem.getDisplayName());
			}
		}
		return null;
	}
	
	private ActionForward search(HttpServletRequest request, HttpServletResponse response,
			HttpSession session, GrouperSession grouperSession, XmlExporter xe, PrintWriter pw)
			throws Exception {
		String term=request.getParameter("searchTerm");
		String type=request.getParameter("searchType");
		if(isEmpty(term)) throw new IllegalArgumentException("Cannot have empty search string");
		Collection results=null;
		if("subjects".equals(type)) {
			results = SubjectFinder.findAll(term);
		}else if("groups".equals(type)){
			GroupAnyAttributeFilter gaaf = new GroupAnyAttributeFilter(term,StemFinder.findRootStem(grouperSession));
			GrouperQuery q = GrouperQuery.createQuery(grouperSession,gaaf);
			results=q.getGroups();
		}else if("stems".equals(type)) {
			StemNameAnyFilter snaf = new StemNameAnyFilter(term,StemFinder.findRootStem(grouperSession));
			GrouperQuery q = GrouperQuery.createQuery(grouperSession,snaf);
			results=q.getStems();
		}else{
			throw new IllegalArgumentException("Invalid searchType:" + type + ": Only stems, groups and subjects accepted");
		}
		xe.export(pw, results, "Searched for `" + term + "` in " + type);
		
		
		return null;
	}
	
	private Map properties2map(Properties props) {
		Map map = new HashMap();
		Set es = props.entrySet();
		Iterator it = es.iterator();
		Map.Entry entry;
		while(it.hasNext()) {
			entry = (Map.Entry) it.next();
			map.put(entry.getKey(), entry.getValue());
		}
		
		return map;
	}

}